/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package business.entities;

import business.base.BaseSubject;
import business.base.IHasCredit;
import business.base.IHasExam;
import business.base.IList;
import business.base.IListItem;
import java.util.HashMap;
import java.util.LinkedHashMap;

/**
 *
 * @author balikm1
 */
public class Result implements IListItem {

    private final BaseSubject subject;
    private boolean exam;
    private boolean credit;
    private int grade;

    public Result(BaseSubject subject) {
        this.subject = subject;
        exam = false;
        credit = false;
    }

    // <editor-fold defaultstate="collapsed" desc="Object overrides">

    @Override
    public boolean equals(Object obj) {
        if (obj == null) {
            return false;
        }
        if (getClass() != obj.getClass()) {
            return false;
        }
        final Result other = (Result) obj;
        if (this.subject != other.subject && (this.subject == null || !this.subject.equals(other.subject))) {
            return false;
        }
        return true;
    }

    @Override
    public int hashCode() {
        int hash = 7;
        hash = 29 * hash + (this.subject != null ? this.subject.hashCode() : 0);
        return hash;
    }

    @Override
    public String toString() {
        return "Result{" + "subject=" + subject + "exam=" + exam + "credit=" + credit + "grade=" + grade + '}';
    }

    // </editor-fold>

    // <editor-fold defaultstate="collapsed" desc="Data manipulation methods">

    public BaseSubject getSubject()
    {
        return subject;
    }

    public boolean hasCredit()
    {
        return credit;
    }

    public void setCredit() throws Exception
    {
        if (subject instanceof IHasCredit)
        {
            credit = true;
        } else {
            throw new Exception(String.format("Subject %s has no credit.", subject.getName()));
        }
    }

    public void removeCredit() throws Exception
    {
        if (subject instanceof IHasCredit)
        {
            credit = false;
        } else {
            throw new Exception(String.format("Subject %s has no credit.", subject.getName()));
        }
    }
    
    public boolean hasExamResult()
    {
        return exam;
    }

    public int getExamResult() throws Exception
    {
        if (!exam)
            throw new Exception(String.format("Subject %s has no exam.", subject.getName()));
        return grade;
    }

    public void setExamResult(int grade) throws Exception
    {
        if (subject instanceof IHasExam)
        {
            exam = true;
            this.grade = grade;
        } else {
            throw new Exception(String.format("Subject %s has no exam.", subject.getName()));
        }
    }

    public void removeExamResult() throws Exception
    {
        if (subject instanceof IHasExam)
        {
            exam = false;
        } else {
            throw new Exception(String.format("Subject %s has no exam.", subject.getName()));
        }
    }

    // </editor-fold>

    // <editor-fold defaultstate="collapsed" desc="IListItem methods">

    public HashMap<String, String> getPublicAttributes() {
        HashMap<String, String> map = new LinkedHashMap<String, String>();
        map.put("Předmět", this.subject.getName());
        if (this.subject instanceof IHasCredit)
        {
            map.put("Zápočet", this.credit ? "Ano" : "Ne");
        }
        if (this.subject instanceof IHasExam)
            map.put("Známka", this.exam ? Integer.toString(grade) : "-");
        return map;
    }
    
    public boolean hasSubitems() {
        return false;
    }

    public IList getSubitems() {
        throw new UnsupportedOperationException("Has no subitems.");
    }

    public boolean isEditable() {
        return true;
    }

    public boolean canCreate() {
        return false;
    }

    // </editor-fold>

}
